/**
  *  \file game/objnote.h
  *  \brief Note Objects
  *
  *  (c) 2001-2021 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_OBJNOTE_H
#define PCC_V2_GAME_OBJNOTE_H

#include "game/obj.h"
#include "game/objtypes.h"
#include "cpluslib/string.h"
#include "cpluslib/ptrvector.h"

class GUniverse;

/** Note Object. This is a pseudo object used to add additional entries to
    object lists. Each of those consists of an Id number and a name.
    - If the Id number is zero, the object is called a <b>divider</b> and
      is treated specially in various ways. In particular, it is not selectable
      (not existant).
    - If the Id number is nonzero, it is a special object not described
      in more detail here, but can be selected normally. This applies, for
      example, for special "cancel" entries in lists. */
class GNoteObject : public GObject {
    int id;
    string_t name;
 public:
    GNoteObject(int id, string_t name);
    GNoteObject(const GNoteObject& other);
    ~GNoteObject();

    string_t getName(Name which) const;
    int      getId() const;
    int      getOwner() const;
};

/** Note Container. Manages a set of GNoteObject's. It acts as GObjectType
    as well as lifetime management for them. */
class GNoteContainer : public GObjectType {
 public:
    GNoteContainer();
    ~GNoteContainer();

    // GObjectType:
    GNoteObject& getObjectByIndex(int index);
    GUniverse*   getUniverseByIndex(int index);
    bool         isValidIndex(int index) const;
    int          getNextIndex(int index) const;
    int          getPreviousIndex(int index) const;

    // GNoteContainer:
    int          add(int id, string_t name);
    int          addCopyOf(const GNoteObject& obj);
    void         clear();
    void         swap(GNoteContainer& other);

 private:
    ptr_vector<GNoteObject> notes;
};

#endif
