/**
  *  \file game/planetform2.cc
  *  \brief Planet Formulas Part 2
  *
  *  Separate from planetform.cc to reduce link dependencies.
  *
  *  (c) 2001-2021 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/planetform.h"
#include "game/config.h"

/*
 *  Starbase Formulas
 */

/** Compute cost for a tech level upgrade.
    Returns the cost for upgrading from %fromTech to %toTech.
    \param player Player to compute this for
    \param fromTech Old (low) tech
    \param toTech New (high) tech
    \return Cost in megacredits */
int32_t
getBaseTechCost(int player, int fromTech, int toTech)
{
    /* Going from tech i to i+1 costs 100*i mc, thus going from 0 to i
       costs 100*\sum[0,i] i = 100*(i*(i-1)/2) = 50*i*(i-1).

       Using BaseTechCost, the formula becomes
           BaseTechCost*(i*(i-1)/2)
       Because either i or i-1 will be even, the division by two will
       always work out. */
    int fromVal = fromTech * (fromTech-1);
    int toVal   = toTech   * (toTech-1);
    return config.BaseTechCost(player) * (toVal - fromVal) / 2;
}
