/**
  *  \file game/planettype.cc
  *  \brief Planet Object Types
  *
  *  (c) 2001-2021 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "game/planettype.h"
#include "game/struct.h"
#include "game/univ.h"
#include "game/planet.h"

static bool
isValidPlanetId(int id)
{
    return id > 0 && id <= NUM_PLANETS;
}

/*************************** GPlayedPlanetType ***************************/

GPlayedPlanetType::GPlayedPlanetType(GUniverse& univ)
    : GIndexedObjectType(NUM_PLANETS),
      univ(univ)
{ }
GPlanet&
GPlayedPlanetType::getObjectByIndex(int index)
{
    return univ.getPlanet(index);
}

GUniverse*
GPlayedPlanetType::getUniverseByIndex(int /*index*/)
{
    return &univ;
}

bool
GPlayedPlanetType::isValidIndex(int index) const
{
    return isValidPlanetId(index) && univ.getPlanet(index).isPlayable(GObject::ReadOnly);
}

/***************************** GAnyPlanetType ****************************/

GAnyPlanetType::GAnyPlanetType(GUniverse& univ)
    : GIndexedObjectType(NUM_PLANETS),
      univ(univ)
{ }

GPlanet&
GAnyPlanetType::getObjectByIndex(int index)
{
    return univ.getPlanet(index);
}

GUniverse*
GAnyPlanetType::getUniverseByIndex(int /*index*/)
{
    return &univ;
}

bool
GAnyPlanetType::isValidIndex(int index) const
{
    return isValidPlanetId(index) && univ.getPlanet(index).isVisible();
}

/**************************** GPlayedBaseType ****************************/

GPlayedBaseType::GPlayedBaseType(GUniverse& univ)
    : GIndexedObjectType(NUM_PLANETS),
      univ(univ)
{ }

GPlanet&
GPlayedBaseType::getObjectByIndex(int index)
{
    return univ.getPlanet(index);
}

GUniverse*
GPlayedBaseType::getUniverseByIndex(int /*index*/)
{
    return &univ;
}

bool
GPlayedBaseType::isValidIndex(int index) const
{
    if (!isValidPlanetId(index)) {
        return false;
    } else {
        const GPlanet& p = univ.getPlanet(index);
        return p.isPlayable(GObject::ReadOnly) && p.hasBase();
    }
}
